@@ -3,6 +3,9 @@ use warnings FATAL => 'recursion';
 use Any::Moose;
 use Any::Moose '::Util::TypeConstraints';
 
+use Scalar::Util ();
+use Carp         ();
+
 use Text::Xslate::Parser;
 use Text::Xslate::Util qw(
     $DEBUG
@@ -12,8 +15,6 @@ use Text::Xslate::Util qw(
     p
 );
 
-use Scalar::Util ();
-
 #use constant _VERBOSE  => scalar($DEBUG =~ /\b verbose \b/xms);
 use constant _DUMP_ASM => scalar($DEBUG =~ /\b dump=asm \b/xms);
 use constant _DUMP_AST => scalar($DEBUG =~ /\b dump=ast \b/xms);
@@ -39,6 +40,8 @@ my %binary = (
     '>'  => 'gt',
     '>=' => 'ge',
 
+    '~~'  => 'match',
+
     '<=>' => 'ncmp',
     'cmp' => 'scmp',
 
@@ -183,6 +186,13 @@ has [qw(header footer)] => (
     isa => 'ArrayRef',
 );
 
+has current_file => (
+    is  => 'rw',
+    isa => 'Str',
+
+    init_arg => undef,
+);
+
 sub lvar_use {
     my($self, $n) = @_;
 
@@ -192,6 +202,8 @@ sub lvar_use {
 sub compile {
     my($self, $input, %args) = @_;
 
+    my $default_filename = '<input>';
+
     # each compiling process is independent
     local $self->{macro_table}  = {};
     local $self->{lvar_id     } = 0;
@@ -199,13 +211,14 @@ sub compile {
     local $self->{const}        = [];
     local $self->{dependencies} = [];
     local $self->{cascade};
-    local $self->{header}  = $self->{header};
-    local $self->{footer}  = $self->{footer};
+    local $self->{header}       = $self->{header};
+    local $self->{footer}       = $self->{footer};
+    local $self->{current_file} = $default_filename;
 
-    my $parser   = $self->parser;
-    my $old_file = $parser->file;
+    my $parser = $self->parser;
+    local $parser->{file};
 
-    $args{file} ||= '<input>';
+    $args{file} ||= $default_filename;
 
     my $header = delete $self->{header};
     my $footer = delete $self->{footer};
@@ -217,12 +230,15 @@ sub compile {
         $input .= $self->_cat_files($footer);
     }
 
-    my @code; # main protocode
+    my @code; # main code
     {
         my $ast = $parser->parse($input, %args);
         print STDERR p($ast) if _DUMP_AST;
-        @code = $self->_compile_ast($ast);
-        $self->_finish_main(\@code);
+        @code = (
+            $self->opcode(set_opinfo => undef, file => $args{file}, line => 1),
+            $self->_compile_ast($ast),
+            $self->opcode('end'),
+        );
     }
 
     my $cascade = $self->cascade;
@@ -240,8 +256,6 @@ sub compile {
         $self->as_assembly(\@code, scalar($DEBUG =~ /\b ix \b/xms))
             if _DUMP_ASM;
 
-    $parser->file($old_file || '<input>'); # reset
-
     {
         my %uniq;
         push @code,
@@ -252,6 +266,28 @@ sub compile {
     return \@code;
 }
 
+sub opcode { # build an opcode
+    my($self, $name, $arg, %args) = @_;
+    my $symbol = $args{symbol};
+    my $file   = $args{file};
+    if(not defined $file) {
+        $file = $self->file;
+        if($file ne $self->current_file) {
+            $self->current_file($file);
+        }
+        else {
+            $file = undef;
+        }
+    }
+    # name, arg, line, file, symbol, comment
+    return [ $name => $arg,
+                $args{line} || (ref $symbol ? $symbol->line : undef),
+                $file,
+                (ref $symbol ? $symbol->id : $symbol),
+                $args{comment},
+           ];
+}
+
 sub _cat_files {
     my($self, $files) = @_;
     my $engine = $self->engine || $self->_error("No Xslate engine which header/footer requires");
@@ -264,12 +300,6 @@ sub _cat_files {
     return $s;
 }
 
-sub _finish_main {
-    my($self, $main_code) = @_;
-    push @{$main_code}, ['end'];
-    return;
-}
-
 our $_lv = -1;
 
 sub _compile_ast {
@@ -340,13 +370,16 @@ sub _process_cascade {
         my $mtable   = $self->macro_table;
         my $macro;
         foreach my $c(@{$code}) {
-            # retrieve macros
+            # $c = [name, arg, line, file, symbol ]
+
+            # retrieve macros from assembly code
             if($c->[0] eq 'macro_begin' .. $c->[0] eq 'macro_end') {
                 if($c->[0] eq 'macro_begin') {
                     $macro = [];
                     $macro = {
                         name  => $c->[1],
                         line  => $c->[2],
+                        file  => $c->[3],
                         body  => [],
                     };
                     push @{ $mtable->{$c->[1]} ||= [] }, $macro;
@@ -365,7 +398,6 @@ sub _process_cascade {
                 }
             }
         }
-
         $self->requires($fullpath);
         $self->_process_cascade_file($cfile, $base_code);
     }
@@ -377,13 +409,8 @@ sub _process_cascade {
         }
 
         foreach my $c(@{$main_code}) {
-            if(!($c->[0] eq 'print_raw_s' && $c->[1] =~ m{\A [ \t\r\n]* \z}xms)) {
-                if($c->[0] eq 'print_raw_s') {
-                    Carp::carp("Xslate: Uselses use of text '$c->[1]'");
-                }
-                else {
-                    #Carp::carp("Xslate: Useless use of $c->[0] " . ($c->[1] // ""));
-                }
+            if($c->[0] eq 'print_raw_s' && $c->[1] =~ m{ [^ \t\r\n] }xms) {
+                Carp::carp("Xslate: Uselses use of text '$c->[1]'");
             }
         }
         @{$main_code} = @{$base_code};
@@ -465,15 +492,17 @@ sub _flush_macro_table {
     foreach my $macros(values %{$mtable}) {
         foreach my $macro(ref($macros) eq 'ARRAY' ? @{$macros} : $macros) {
             push @code,
-                [ macro_begin => $macro->{name}, $macro->{line} ];
+                $self->opcode( macro_begin => $macro->{name},
+                    file => $macro->{file},
+                    line => $macro->{line} );
 
-            push @code, [ macro_nargs => $macro->{nargs} ]
+            push @code, $self->opcode( macro_nargs => $macro->{nargs} )
                 if $macro->{nargs};
 
-            push @code, [ macro_outer => $macro->{outer} ]
+            push @code, $self->opcode( macro_outer => $macro->{outer} )
                 if $macro->{outer};
 
-            push @code, @{ $macro->{body} }, [ macro_end => $macro->{immediate} ];
+            push @code, @{ $macro->{body} }, $self->opcode( macro_end => $macro->{immediate} );
         }
     }
     %{$mtable} = ();
@@ -492,11 +521,11 @@ sub _generate_name {
             return @{$code};
         }
         else {
-            return [ load_lvar => $lvar_id, $node->line, "constant $node" ];
+            return $self->opcode( load_lvar => $lvar_id, symbol => $node );
         }
     }
 
-    return [ symbol => $node->id, $node->line ];
+    return $self->opcode( symbol => $node->id, line => $node->line );
 }
 
 sub _can_print_optimize {
@@ -523,25 +552,26 @@ sub _generate_command {
 
     foreach my $arg(@{ $node->first }){
         if(exists $Text::Xslate::OPS{$proc . '_s'} && $arg->arity eq 'literal'){
-            push @code, [ $proc . '_s' => literal_to_value($arg->value), $node->line ];
+            push @code, $self->opcode( $proc . '_s' => $arg->value );
         }
         elsif($self->_can_print_optimize($proc, $arg)){
-            my $filter_name = $arg->first->id;
+            my $filter      = $arg->first;
+            my $filter_name = $filter->id;
             my $command = $builtin{ $filter_name } eq 'builtin_mark_raw'
                 ? 'print_raw'  # mark_raw, raw
                 : 'print';     # html
             push @code,
                 $self->_expr($arg->second->[0]),
-                [ $command => undef, $node->line, "$filter_name (builtin filter)" ];
+                $self->opcode( $command => undef, symbol => $filter );
         }
         else {
             push @code,
                 $self->_expr($arg),
-                [ $proc => undef, $node->line ];
+                $self->opcode( $proc => undef, line => $node->line );
         }
     }
     if(defined(my $vars = $node->second)) {
-        @code = (['enter'], $self->_localize_vars($vars), @code, ['leave']);
+        @code = ($self->opcode('enter'), $self->_localize_vars($vars), @code, $self->opcode('leave'));
     }
 
     if(!@code) {
@@ -553,12 +583,11 @@ sub _generate_command {
 sub _bare_to_file {
     my($self, $file) = @_;
     if(ref($file) eq 'ARRAY') { # myapp::foo
-        $file  = join('/', @{$file}) . $self->{engine}->{suffix};
+        return join('/', map { $_->value } @{$file}) . $self->{engine}->{suffix};
     }
-    else { # "myapp/foo.tx"
-        $file = literal_to_value($file);
+    else {
+        return $file->value;
     }
-    return $file;
 }
 
 sub _generate_cascade {
@@ -578,8 +607,8 @@ sub _compile_block {
         if($op->[0] eq 'pushmark') {
             # pushmark ... funcall (or something) may create mortal SVs
             # so surround the block with ENTER and LEAVE
-            unshift @block_code, ['enter'];
-            push    @block_code, ['leave'];
+            unshift @block_code, $self->opcode('enter');
+            push    @block_code, $self->opcode('leave');
             last;
         }
     }
@@ -607,17 +636,17 @@ sub _generate_for {
 
     $self->lvar->{$lvar_name} = $lvar_id;
 
-    push @code, [ for_start => $lvar_id, $expr->line, $lvar_name ];
+    push @code, $self->opcode( for_start => $lvar_id, symbol => $iter_var );
 
     # a for statement uses three local variables (container, iterator, and item)
     local $self->{lvar_id} = $self->lvar_use(3);
 
     my @block_code = $self->_compile_block($block);
     push @code,
-        [ literal_i => $lvar_id, $expr->line, $lvar_name ],
-        [ for_iter  => scalar(@block_code) + 2 ],
+        $self->opcode( literal_i => $lvar_id, symbol => $iter_var ),
+        $self->opcode( for_iter  => scalar(@block_code) + 2 ),
         @block_code,
-        [ goto      => -(scalar(@block_code) + 2), undef, "end for" ];
+        $self->opcode( goto      => -(scalar(@block_code) + 2), comment => "end for" );
 
     return @code;
 }
@@ -646,15 +675,15 @@ sub _generate_while {
         $lvar_id                  = $self->lvar_id;
         $lvar_name                = $iter_var->id;
         $self->lvar->{$lvar_name} = $lvar_id;
-        push @code, [ save_to_lvar => $lvar_id, $expr->line, $lvar_name ];
+        push @code, $self->opcode( save_to_lvar => $lvar_id, symbol => $iter_var );
     }
 
     local $self->{lvar_id} = $self->lvar_use(scalar @{$vars});
     my @block_code = $self->_compile_block($block);
     return @code,
-        [ $cond_op => scalar(@block_code) + 2, undef, "while" ],
+        $self->opcode( $cond_op => scalar(@block_code) + 2, symbol => $node ),
         @block_code,
-        [ goto => -(scalar(@block_code) + scalar(@code) + 1), undef, "end while" ];
+        $self->opcode( goto => -(scalar(@block_code) + scalar(@code) + 1), comment => "end while" );
 
     return @code;
 }
@@ -679,14 +708,15 @@ sub _generate_proc { # definition of macro, block, before, around, after
 
     local $self->{lvar_id} = $self->lvar_use($arg_ix);
 
+    my $opinfo = $self->opcode(set_opinfo => undef, file => $self->file, line => $node->line);
     my %macro = (
         name      => $name,
         nargs     => $arg_ix,
-        body      => [ $self->_compile_ast($block) ],
-        line      => $node->line,
-        file      => $self->file,
+        body      => [ $opinfo, $self->_compile_ast($block) ],
+        line      => $opinfo->[2],
+        file      => $opinfo->[3],
         outer     => $lvar_used,
-        immediate => 0,
+        immediate => 0, # whether it returns normal string or raw string
     );
 
     if(any_in($type, qw(macro block))) {
@@ -706,9 +736,6 @@ sub _generate_proc { # definition of macro, block, before, around, after
         $macro{name} = $fq_name;
         push @{ $self->macro_table->{ $fq_name } ||= [] }, \%macro;
     }
-
-    $macro{body} = [ $self->_compile_ast($block) ];
-
     return;
 }
 
@@ -717,7 +744,7 @@ sub _generate_lambda {
 
     my $macro = $node->first;
     $self->_compile_ast([$macro]);
-    return [ symbol => $macro->first->id, $node->line, 'lambda' ];
+    return $self->opcode( symbol => $macro->first->id, line => $node->line );
 }
 
 sub _prepare_cond_expr {
@@ -785,23 +812,23 @@ sub _generate_if {
     if( (@then and @else) or !$OPTIMIZE) {
         return(
             @cond,
-            [ $cond_true => scalar(@then) + 2, undef, $node->id . ' (then)' ],
+            $self->opcode( $cond_true => scalar(@then) + 2, comment => $node->id . ' (then)' ),
             @then,
-            [ goto => scalar(@else) + 1, undef, $node->id . ' (else)' ],
+            $self->opcode( goto => scalar(@else) + 1, comment => $node->id . ' (else)' ),
             @else,
         );
     }
     elsif(!@else) { # no @else
         return(
             @cond,
-            [ $cond_true => scalar(@then) + 1, undef, $node->id . ' (then/no-else)' ],
+            $self->opcode( $cond_true => scalar(@then) + 1, comment => $node->id . ' (then/no-else)' ),
             @then,
         );
     }
     else { # no @then
         return(
             @cond,
-            [ $cond_false => scalar(@else) + 1, undef, $node->id . ' (else/no-then)'],
+            $self->opcode( $cond_false => scalar(@else) + 1, comment => $node->id . ' (else/no-then)'),
             @else,
         );
     }
@@ -828,7 +855,7 @@ sub _generate_given {
     $self->lvar->{$lvar_name} = $lvar_id;
 
     local $self->{lvar_id} = $self->lvar_use(1); # topic variable
-    push @code, [ save_to_lvar => $lvar_id, undef, "given $lvar_name" ],
+    push @code, $self->opcode( save_to_lvar => $lvar_id, symbol => $lvar ),
         $self->_compile_ast($block);
 
     return @code;
@@ -837,37 +864,33 @@ sub _generate_given {
 sub _generate_variable {
     my($self, $node) = @_;
 
-    my @op;
     if(defined(my $lvar_id = $self->lvar->{$node->id})) {
-        @op = ( load_lvar => $lvar_id );
+        return $self->opcode( load_lvar => $lvar_id, symbol => $node );
     }
     else {
         my $name = $self->_variable_to_value($node);
         if($name =~ /~/) {
             $self->_error("Undefined iterator variable $node", $node);
         }
-        @op = ( fetch_s => $name );
+        return $self->opcode( fetch_s => $name, line => $node->line );
     }
-    $op[2] = $node->line;
-    $op[3] = $node->id;
-    return \@op;
 }
 
 sub _generate_marker {
     my($self, $node) = @_;
 
-    return [ $node->id => undef, $node->line ];
+    return return $self->opcode( $node->id => undef, symbol => $node );
 }
 
 sub _generate_literal {
     my($self, $node) = @_;
 
-    my $value = literal_to_value($node->value);
+    my $value = $node->value;
     if(defined $value){
-        return [ literal => $value ];
+        return $self->opcode( literal => $value );
     }
     else {
-        return [ nil => undef ];
+        return $self->opcode( 'nil' );
     }
 }
 
@@ -878,9 +901,9 @@ sub _generate_objectliteral {
     my $type = $node->id eq '{' ? 'make_hash' : 'make_array';
 
     return
-        ['pushmark', undef, undef, $type],
+        $self->opcode( pushmark => undef, comment => $type ),
         (map{ $self->_expr($_), ['push'] } @{$list}),
-        [$type],
+        $self->opcode($type),
     ;
 }
 
@@ -889,7 +912,10 @@ sub _generate_unary {
 
     my $id = $node->id;
     if(exists $unary{$id}) {
-        my @code = ($self->_expr($node->first), [ $unary{$id} ]);
+        my @code = (
+            $self->_expr($node->first),
+            $self->opcode( $unary{$id} )
+        );
         if( $OPTIMIZE and $self->_code_is_literal($code[0]) ) {
             $self->_fold_constants(\@code);
         }
@@ -909,7 +935,7 @@ sub _generate_binary {
     if($id eq '.') {
         return
             @lhs,
-            [ fetch_field_s => $node->second->id ];
+            $self->opcode( fetch_field_s => $node->second->id );
     }
     elsif(exists $binary{$id}) {
         if($id eq "["
@@ -918,28 +944,28 @@ sub _generate_binary {
             # $foo[literal]
             return
                 @lhs,
-                [ fetch_field_s => literal_to_value($node->second->value) ];
+                $self->opcode( fetch_field_s => $node->second->value );
         }
 
         local $self->{lvar_id} = $self->lvar_use(1);
         my @rhs = $self->_expr($node->second);
         my @code = (
             @lhs,
-            [ save_to_lvar => $self->lvar_id ],
+            $self->opcode( save_to_lvar => $self->lvar_id ),
             @rhs,
-            [ load_lvar_to_sb => $self->lvar_id ],
-            [ $binary{$id}   => undef ],
+            $self->opcode( load_lvar_to_sb => $self->lvar_id ),
+            $self->opcode( $binary{$id} ),
         );
 
         if(any_in($id, qw(min max))) {
             local $self->{lvar_id} = $self->lvar_use(1);
             splice @code, -1, 0,
-                [save_to_lvar => $self->lvar_id ]; # save lhs
+                $self->opcode(save_to_lvar => $self->lvar_id ); # save lhs
             push @code,
-                [ or              => +2 , undef, $id ],
-                [ load_lvar_to_sb => $self->lvar_id, undef, "$id on false" ],
+                $self->opcode( or => +2 , symbol => $node ),
+                $self->opcode( load_lvar_to_sb => $self->lvar_id ), # on true
                 # fall through
-                [ move_from_sb    => undef, undef, "$id on true" ],
+                $self->opcode( 'move_from_sb' ), # on false
         }
 
         if($OPTIMIZE) {
@@ -953,7 +979,7 @@ sub _generate_binary {
         my @rhs = $self->_expr($node->second);
         return
             @lhs,
-            [ $logical_binary{$id} => scalar(@rhs) + 1, undef, "logical $id" ],
+            $self->opcode( $logical_binary{$id} => scalar(@rhs) + 1, symbol => $node ),
             @rhs;
     }
 
@@ -966,11 +992,11 @@ sub _generate_methodcall {
     my $args   = $node->third;
     my $method = $node->second->id;
     return (
-        [ pushmark => undef, undef, $method ],
+        $self->opcode( pushmark => undef, comment => $method ),
         $self->_expr($node->first),
-        [ 'push' ],
-        (map { $self->_expr($_), [ 'push' ] } @{$args}),
-        [ methodcall_s => $method, $node->line ],
+        $self->opcode( 'push' ),
+        (map { $self->_expr($_), $self->opcode('push') } @{$args}),
+        $self->opcode( methodcall_s => $method, line => $node->line ),
     );
 }
 
@@ -987,10 +1013,10 @@ sub _generate_call {
     }
 
     return(
-        [ pushmark => undef, undef, "funcall " . $callable->id ],
-        (map { $self->_expr($_), [ 'push' ] } @{$args}),
+        $self->opcode( pushmark => undef, comment => $callable->id ),
+        (map { $self->_expr($_), $self->opcode( 'push' ) } @{$args}),
         $self->_expr($callable),
-        [ funcall => undef, $node->line ],
+        $self->opcode( 'funcall' )
     );
 }
 
@@ -1005,7 +1031,7 @@ sub _generate_iterator {
             $node);
     }
 
-    return [ load_lvar => $lvar_id+1, $node->line, $node->id ];
+    return $self->opcode( load_lvar => $lvar_id+1, symbol => $node );
 }
 
 sub _generate_iterator_body {
@@ -1018,7 +1044,7 @@ sub _generate_iterator_body {
             $node);
     }
 
-    return [ load_lvar => $lvar_id+2, $node->line, $node->id ];
+    return $self->opcode( load_lvar => $lvar_id+2, symbol => $node );
 }
 
 sub _generate_assign {
@@ -1047,7 +1073,7 @@ sub _generate_assign {
 
     return
         @expr,
-        [ save_to_lvar => $lvar->{$lvar_name}, $node->line, $lvar_name ];
+        $self->opcode( save_to_lvar => $lvar->{$lvar_name}, symbol => $lhs, comment => $node->id);
 }
 
 sub _generate_constant {
@@ -1073,7 +1099,7 @@ sub _generate_constant {
 
     return
         @expr,
-        [ save_to_lvar => $lvar_id, $node->line, "constant $lvar_name" ];
+        $self->opcode( save_to_lvar => $lvar_id, symbol => $lhs, comment => $node->id);
 }
 
 
@@ -1087,7 +1113,7 @@ sub _localize_vars {
         }
         push @localize,
             $self->_expr($expr),
-            [ localize_s => literal_to_value($key->value) ];
+            $self->opcode( localize_s => $key->value, symbol => $key );
     }
     return @localize;
 }
@@ -1122,7 +1148,8 @@ sub _fold_constants {
     local $engine->{verbose}      = 1;
 
     my $result = eval {
-        $engine->_assemble([@{$code}, ['print_raw'], ['end']], undef, undef, undef, undef);
+        my @tmp_code = (@{$code}, $self->opcode('print_raw'), $self->opcode('end'));
+        $engine->_assemble(\@tmp_code, undef, undef, undef, undef);
         $engine->render(undef);
     };
     if($@) {
@@ -1130,7 +1157,7 @@ sub _fold_constants {
         return 0;
     }
 
-    @{$code} = ([ literal => $result, undef, "optimized by constant folding"]);
+    @{$code} = ($self->opcode( literal => $result, comment => "optimized by constant folding"));
     return 1;
 }
 
@@ -1145,8 +1172,8 @@ my %goto_family;
 )} = ();
 
 sub _noop {
-    my($op) = @_;
-    @{$op} = (noop => undef, undef, "ex-$op->[0]");
+    my($self, $op) = @_;
+    @{$op} = @{ $self->opcode( noop => undef, comment => "ex-$op->[0]") };
     return;
 }
 
@@ -1194,7 +1221,7 @@ sub _optimize_vmcode {
 
                 $c->[$i][1] .= $c->[$j][1];
 
-                _noop($c->[$j]);
+                $self->_noop($c->[$j]);
                 $j++;
             }
         }
@@ -1213,9 +1240,9 @@ sub _optimize_vmcode {
             if(defined($nn)
                 && $nn->[0] eq 'load_lvar_to_sb'
                 && $nn->[1] == $it->[1]) {
-                @{$it} = ('move_to_sb', undef, undef, "ex-$it->[0]");
+                @{$it} = @{$self->opcode( move_to_sb => undef, comment => "ex-$it->[0]" )};
 
-                _noop($nn);
+                $self->_noop($nn);
             }
         }
         elsif($name eq 'literal') {
@@ -1227,9 +1254,9 @@ sub _optimize_vmcode {
             my $prev = $c->[$i-1];
             if($prev->[0] =~ /^literal/) { # literal or literal_i
                 $c->[$i][0] = 'fetch_field_s';
-                $c->[$i][1] = $prev->[1];
+                $c->[$i][1] = $prev->[1]; # arg
 
-                _noop($prev);
+                $self->_noop($prev);
             }
         }
     }
@@ -1256,25 +1283,32 @@ sub _optimize_vmcode {
 sub as_assembly {
     my($self, $code_ref, $addix) = @_;
 
-    my $as = "";
+    my $asm = "";
     foreach my $ix(0 .. (@{$code_ref}-1)) {
-        my($opname, $arg, $line, $comment) = @{$code_ref->[$ix]};
-        $as .= "$ix:" if $addix; # for debugging
+        my($name, $arg, $line, $file, $symbol, $comment) = @{$code_ref->[$ix]};
+        $asm .= "$ix:" if $addix; # for debugging
 
-        # "$opname $arg #$line // $comment"
-        $as .= $opname;
+        # "$opname $arg #$line:$file *$symbol // $comment"
+        ref($name) and die "Oops: " . p($code_ref->[$ix]);
+        $asm .= $name;
         if(defined $arg) {
-            $as .= " " . value_to_literal($arg);
+            $asm .= " " . value_to_literal($arg);
         }
         if(defined $line) {
-            $as .= " #$line";
+            $asm .= " #$line";
+            if(defined $file) {
+                $asm .= ":" . value_to_literal($file);
+            }
+        }
+        if(defined $symbol) {
+            $asm .= " *" . value_to_literal($symbol);
         }
         if(defined $comment) {
-            $as .= " // $comment";
+            $asm .= " // $comment";
         }
-        $as .= "\n";
+        $asm .= "\n";
     }
-    return $as;
+    return $asm;
 }
 
 sub _error {
