@@ -22,6 +22,7 @@ around trim_code => sub {
 
 sub init_symbols {
     my($parser) = @_;
+    my $s;
 
     $parser->symbol(']');
     $parser->symbol('}');
@@ -54,6 +55,15 @@ sub init_symbols {
     $parser->symbol('WHILE')   ->set_std(\&std_while);
     $parser->symbol('while')   ->set_std(\&std_while);
 
+    $parser->symbol('SWITCH')   ->set_std(\&std_switch);
+    $parser->symbol('switch')   ->set_std(\&std_switch);
+    $s = $parser->symbol('CASE');
+    $s->set_std(\&std_case);
+    $s->is_block_end(1);
+    $s = $parser->symbol('case');
+    $s->set_std(\&std_case);
+    $s->is_block_end(1);
+
     $parser->symbol('INCLUDE') ->set_std(\&std_include);
     $parser->symbol('include') ->set_std(\&std_include);
     $parser->symbol('WITH');
@@ -227,6 +237,50 @@ sub std_if {
     return $top_if;
 }
 
+sub std_switch {
+    my($parser, $symbol) = @_;
+
+    $parser->new_scope();
+
+    my $topic  = $parser->symbol('$_')->clone(arity => 'variable' );
+    my $switch = $symbol->clone(
+        arity  => 'given',
+        first  => $parser->expression(0),
+        second => [ $topic ],
+    );
+
+    local $parser->{in_given} = 1;
+
+    my @cases;
+    while(uc($parser->token->id) ne "END") {
+        push @cases, $parser->statement();
+    }
+    $switch->third( \@cases );
+
+    $parser->build_given_body($switch, "case");
+
+    $parser->advance("END");
+    $parser->pop_scope();
+    return $switch;
+}
+
+sub std_case {
+    my($parser, $symbol) = @_;
+    if(!$parser->in_given) {
+        $parser->_error("You cannot use $symbol statements outside switch statements");
+    }
+    my $case = $symbol->clone(arity => "case");
+
+    if(uc($parser->token->id) ne "DEFAULT") {
+        $case->first( $parser->expression(0) );
+    }
+    else {
+        $parser->advance();
+    }
+    $case->second( $parser->statements() );
+    return $case;
+}
+
 sub iterator_name {
     return 'loop'; # always 'loop'
 }
@@ -447,7 +501,7 @@ sub wrap {
     $content->second([]); # args
     $content->third($body);
 
-    my $call_content = $parser->call($proto, $content->first);
+    my $call_content = $parser->call($content->first);
 
     my $into_name = $proto->clone(
         arity => 'literal',
@@ -463,9 +517,9 @@ sub wrap {
 # ...
 # [% END %]
 # is
-# : __block filter_xxx -> {
+# : block filter_xxx | html -> {
 #   ...
-# : } filter_001() | html
+# : }
 # in Kolon
 
 sub std_filter {
@@ -487,8 +541,8 @@ sub std_filter {
 
     # _immediate_block() | filter
 
-    my $callmacro  = $parser->call($symbol, $proc->first);
-    my $callfilter = $parser->call($symbol, $filter, $callmacro);
+    my $callmacro  = $parser->call($proc->first);
+    my $callfilter = $parser->call($filter, $callmacro);
 
     my $print = $parser->symbol('print')->clone(
         arity => 'command',
@@ -519,9 +573,11 @@ Text::Xslate::Syntax::TTerse - An alternative syntax compatible with Template To
         { dialect => 'TTerse' }
     );
 
-    # PRE_PROCESS/POST_PROCESS/WRAPPER
+    # PRE_PROCESS/POST_PROCESS
     $tx = Text::Xslate->new(
         syntax => 'TTerse',
+        header => ['header.tt'],
+        footer => ['footer.tt'],
     );
 
 =head1 DESCRIPTION
@@ -565,7 +621,7 @@ If I<$var> is an object instance, you can call its methods.
 
 =head2 Expressions
 
-Almost the same as L<Text::Xslate::Syntax::Kolon>, but the C<_> operator for
+Almost the same as L<Text::Xslate::Syntax::Kolon>, but C<< infix:<_> >> for
 concatenation is supported for compatibility.
 
 =head2 Loops
@@ -603,20 +659,29 @@ discouraged because they are not easy to understand:
 
 =head2 Conditional statements
 
-    [% IF expression %]
-        This is true
-    [% ELSE %]
-        Tis is false
-    [% END %]
-
-    [% IF expression %]
+    [% IF logical_expression %]
         Case 1
-    [% ELSIF expression %]
+    [% ELSIF logical_expression %]
         Case 2
     [% ELSE %]
         Case 3
     [% END %]
 
+    [% UNLESS logical_expression %]
+        Case 1
+    [% ELSE %]
+        Case 2
+    [% END %]
+
+    [% SWITCH expression %]
+    [% CASE case1 %]
+        Case 1
+    [% CASE case2 %]
+        Case 2
+    [% CASE DEFAULT %]
+        Case 3
+    [% END %]
+
 =head2 Functions and filters
 
     [% var | f %]
@@ -708,13 +773,12 @@ implementation.
     use Text::Xslate::Bridge::TT2;
 
     my $tx = Text::Xslate->new(
-        module => [qw(Text::Xslate:*Bridge::TT2)],
+        module => [qw(Text::Xslate::Bridge::TT2)],
     );
 
    print $tx->render_strig('[% "foo".length() %]'); # => 3
 
-See L<Text::Xslate::Bridge>, L<Text::Xslate::Bridge::TT2>, and
-L<Text::Xslate::Bridge::Alloy> for details.
+See L<Text::Xslate::Bridge>, or search for C<Text::Xslate::Bridge::*> on CPAN.
 
 =head2 Misc.
 
@@ -746,4 +810,10 @@ L<Template::Toolkit>
 
 L<Template::Tiny>
 
+L<Text::Xslate::Bridge::TT2>
+
+L<Text::Xslate::Bridge::TT2::Like>
+
+L<Text::Xslate::Bridge::Alloy>
+
 =cut
